//+------------------------------------------------------------------+
//|                                             SignalCustomMACD.mqh |
//|                      Copyright  2010, MetaQuotes Software Corp. |
//|                                        http://www.metaquotes.net |
//|                                              Revision 2011.01.25 |
//+------------------------------------------------------------------+
#include <Expert\ExpertSignal.mqh>
// wizard description start
//+------------------------------------------------------------------+
//| Description of the class                                         |
//| Title=Signals based on crossover of main and signal MACD lines   |
//| with custom inicator                                             |
//| Type=Signal                                                      |
//| Name=MACD                                                        |
//| Class=CSignalCustomMACD                                          |
//| Page=                                                            |
//| Parameter=PeriodFast,int,12                                      |
//| Parameter=PeriodSlow,int,24                                      |
//| Parameter=PeriodSignal,int,9                                     |
//| Parameter=StopLoss,int,20                                        |
//| Parameter=TakeProfit,int,50                                      |
//+------------------------------------------------------------------+
// wizard description end
//+------------------------------------------------------------------+
//| Class CSignalCustomMACD.                                         |
//| Appointment: Class trading signals                               |
//|              cross of the main and signal lines MACD.            |
//|              Derives from class CExpertSignal.                   |
//+------------------------------------------------------------------+
class CSignalCustomMACD : public CExpertSignal
  {
protected:
   CiCustom          m_MACD;
   //--- input parameters
   int               m_period_fast;
   int               m_period_slow;
   int               m_period_signal;
   int               m_stop_loss;
   int               m_take_profit;

public:
                     CSignalCustomMACD();
                    ~CSignalCustomMACD();
   //--- methods initialize protected data
   void              PeriodFast(int period_fast)     { m_period_fast=period_fast;             }
   void              PeriodSlow(int period_slow)     { m_period_slow=period_slow;             }
   void              PeriodSignal(int period_signal) { m_period_signal=period_signal;         }
   void              StopLoss(int stop_loss)         { m_stop_loss=stop_loss;                 }
   void              TakeProfit(int take_profit)     { m_take_profit=take_profit;             }
   virtual bool      InitIndicators(CIndicators* indicators);
   virtual bool      ValidationSettings();
   //---
   virtual bool      CheckOpenLong(double& price,double& sl,double& tp,datetime& expiration);
   virtual bool      CheckCloseLong(double& price);
   virtual bool      CheckOpenShort(double& price,double& sl,double& tp,datetime& expiration);
   virtual bool      CheckCloseShort(double& price);

protected:
   double            MainMACD(int ind)               { return(m_MACD.GetData(0,ind));         }
   double            SignalMACD(int ind)             { return(m_MACD.GetData(1,ind));         }
   double            StateMACD(int ind)              { return(MainMACD(ind)-SignalMACD(ind)); }
   int               ExtStateMACD(int ind);
  };
//+------------------------------------------------------------------+
//| Constructor CSignalCustomMACD.                                   |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void CSignalCustomMACD::CSignalCustomMACD()
  {
//--- set default inputs
   m_period_fast  =12;
   m_period_slow  =24;
   m_period_signal=9;
   m_stop_loss    =20;
   m_take_profit  =50;
  }
//+------------------------------------------------------------------+
//| Destructor CSignalCustomMACD.                                    |
//| INPUT:  no.                                                      |
//| OUTPUT: no.                                                      |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
void CSignalCustomMACD::~CSignalCustomMACD()
  {
//---
  }
//+------------------------------------------------------------------+
//| Validation settings protected data.                              |
//| INPUT:  no.                                                      |
//| OUTPUT: true-if settings are correct, false otherwise.           |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CSignalCustomMACD::ValidationSettings()
  {
   if(m_period_fast>=m_period_slow)
     {
      printf(__FUNCTION__+": slow period must be greater than fast period");
      return(false);
     }
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Create indicators.                                               |
//| INPUT:  indicators -pointer of indicator collection.             |
//| OUTPUT: true-if successful, false otherwise.                     |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CSignalCustomMACD::InitIndicators(CIndicators* indicators)
  {
//--- check
   if(indicators==NULL) return(false);
//--- add MACD indicator to collection
   if(!indicators.Add(GetPointer(m_MACD)))
     {
      printf(__FUNCTION__+": error adding object");
      return(false);
     }
//--- initialize MACD indicator
   MqlParam params[5];
//--- prepare data for initialize MACD indicator
   params[0].type         =TYPE_STRING;
   params[0].string_value ="Examples\\MACD";
   params[1].type         =TYPE_INT;
   params[1].integer_value=m_period_fast;
   params[2].type         =TYPE_INT;
   params[2].integer_value=m_period_slow;
   params[3].type         =TYPE_INT;
   params[3].integer_value=m_period_signal;
   params[4].type         =TYPE_INT;
   params[4].integer_value=PRICE_CLOSE;
//--- initialize SilverChannels indicator
   if(!m_MACD.NumBuffers(2))
     {
      printf(__FUNCTION__+": error beffers settings");
      return(false);
     }
   if(!m_MACD.Create(m_symbol.Name(),m_period,IND_CUSTOM,5,params))
     {
      printf(__FUNCTION__+": error initializing object");
      return(false);
     }
//--- ok
   return(true);
  }
//+------------------------------------------------------------------+
//| Check relative positions of the main and signal lines MACD.      |
//| INPUT:  ind - start index for check.                             |
//| OUTPUT: absolute value - the number of intervals                 |
//|                     from cross of the main and signal lines MACD,|
//|         sign: minus - fast MA crosses slow MA down,              |
//|               plus - fast MA crosses slow MA upward.             |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
int CSignalCustomMACD::ExtStateMACD(int ind)
  {
   int    res=0;
   double var;
//---
   for(int i=ind;i<5;i++)
     {
      if(MainMACD(i)==WRONG_VALUE || SignalMACD(i)==WRONG_VALUE) break;
      var=StateMACD(i);
      if(res>0)
        {
         if(var<0) break;
         res++;
         continue;
        }
      if(res<0)
        {
         if(var>0) break;
         res--;
         continue;
        }
      if(var>0) res++;
      if(var<0) res--;
     }
//---
   return(res);
  }
//+------------------------------------------------------------------+
//| Check conditions for long position open.                         |
//| INPUT:  price      - refernce for price,                         |
//|         sl         - refernce for stop loss,                     |
//|         tp         - refernce for take profit,                   |
//|         expiration - refernce for expiration.                    |
//| OUTPUT: true-if condition performed, false otherwise.            |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CSignalCustomMACD::CheckOpenLong(double& price,double& sl,double& tp,datetime& expiration)
  {
   price=0.0;
   sl   =m_symbol.Ask()-m_stop_loss*m_adjusted_point;
   tp   =m_symbol.Ask()+m_take_profit*m_adjusted_point;
//---
   return(ExtStateMACD(1)==1);
  }
//+------------------------------------------------------------------+
//| Check conditions for long position close.                        |
//| INPUT:  price - refernce for price.                              |
//| OUTPUT: true-if condition performed, false otherwise.            |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CSignalCustomMACD::CheckCloseLong(double& price)
  {
   price=0.0;
//---
   return(ExtStateMACD(1)==-1);
  }
//+------------------------------------------------------------------+
//| Check conditions for short position open.                        |
//| INPUT:  price      - refernce for price,                         |
//|         sl         - refernce for stop loss,                     |
//|         tp         - refernce for take profit,                   |
//|         expiration - refernce for expiration.                    |
//| OUTPUT: true-if condition performed, false otherwise.            |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CSignalCustomMACD::CheckOpenShort(double& price,double& sl,double& tp,datetime& expiration)
  {
   price=0.0;
   sl   =m_symbol.Bid()+m_stop_loss*m_adjusted_point;
   tp   =m_symbol.Bid()-m_take_profit*m_adjusted_point;
//---
   return(ExtStateMACD(1)==-1);
  }
//+------------------------------------------------------------------+
//| Check conditions for short position close.                       |
//| INPUT:  price - refernce for price.                              |
//| OUTPUT: true-if condition performed, false otherwise.            |
//| REMARK: no.                                                      |
//+------------------------------------------------------------------+
bool CSignalCustomMACD::CheckCloseShort(double& price)
  {
   price=0.0;
//---
   return(ExtStateMACD(1)==1);
  }
//+------------------------------------------------------------------+
